import datetime
import json
import random
import subprocess
import socket
import sys
import time
from bs4 import BeautifulSoup
import requests
import os
from dateutil import parser
from urllib.parse import urlparse
from fake_useragent import UserAgent
try:
  from selenium import webdriver
  from selenium.webdriver.support.wait import WebDriverWait
except ImportError: 
  webdriver = None
  WebDriverWait = None

IN_DOCKER = os.environ.get('IN_DOCKER', False)
ua = UserAgent()
images_path = '/life/data/nga/images/'
htdocs_path = '/life/data/nga/htdocs/'
VERSION = "1.0.5"
local_ip = ""

class FatalScraperError(Exception):
  pass

class DateTimeEncoder(json.JSONEncoder):
  #Override the default method
  def default(self, obj):
    if isinstance(obj, (datetime.date, datetime.datetime)):
      return obj.isoformat()

whois_timezone_info = {
  "A": 1 * 3600,
  "ACDT": 10.5 * 3600,
  "ACST": 9.5 * 3600,
  "ACT": -5 * 3600,
  "ACWST": 8.75 * 3600,
  "ADT": 4 * 3600,
  "AEDT": 11 * 3600,
  "AEST": 10 * 3600,
  "AET": 10 * 3600,
  "AFT": 4.5 * 3600,
  "AKDT": -8 * 3600,
  "AKST": -9 * 3600,
  "ALMT": 6 * 3600,
  "AMST": -3 * 3600,
  "AMT": -4 * 3600,
  "ANAST": 12 * 3600,
  "ANAT": 12 * 3600,
  "AQTT": 5 * 3600,
  "ART": -3 * 3600,
  "AST": 3 * 3600,
  "AT": -4 * 3600,
  "AWDT": 9 * 3600,
  "AWST": 8 * 3600,
  "AZOST": 0 * 3600,
  "AZOT": -1 * 3600,
  "AZST": 5 * 3600,
  "AZT": 4 * 3600,
  "AoE": -12 * 3600,
  "B": 2 * 3600,
  "BNT": 8 * 3600,
  "BOT": -4 * 3600,
  "BRST": -2 * 3600,
  "BRT": -3 * 3600,
  "BST": 6 * 3600,
  "BTT": 6 * 3600,
  "C": 3 * 3600,
  "CAST": 8 * 3600,
  "CAT": 2 * 3600,
  "CCT": 6.5 * 3600,
  "CDT": -5 * 3600,
  "CEST": 2 * 3600,
  "CET": 1 * 3600,
  "CHADT": 13.75 * 3600,
  "CHAST": 12.75 * 3600,
  "CHOST": 9 * 3600,
  "CHOT": 8 * 3600,
  "CHUT": 10 * 3600,
  "CIDST": -4 * 3600,
  "CIST": -5 * 3600,
  "CKT": -10 * 3600,
  "CLST": -3 * 3600,
  "CLT": -4 * 3600,
  "COT": -5 * 3600,
  "CST": -6 * 3600,
  "CT": -6 * 3600,
  "CVT": -1 * 3600,
  "CXT": 7 * 3600,
  "ChST": 10 * 3600,
  "D": 4 * 3600,
  "DAVT": 7 * 3600,
  "DDUT": 10 * 3600,
  "E": 5 * 3600,
  "EASST": -5 * 3600,
  "EAST": -6 * 3600,
  "EAT": 3 * 3600,
  "ECT": -5 * 3600,
  "EDT": -4 * 3600,
  "EEST": 3 * 3600,
  "EET": 2 * 3600,
  "EGST": 0 * 3600,
  "EGT": -1 * 3600,
  "EST": -5 * 3600,
  "ET": -5 * 3600,
  "F": 6 * 3600,
  "FET": 3 * 3600,
  "FJST": 13 * 3600,
  "FJT": 12 * 3600,
  "FKST": -3 * 3600,
  "FKT": -4 * 3600,
  "FNT": -2 * 3600,
  "G": 7 * 3600,
  "GALT": -6 * 3600,
  "GAMT": -9 * 3600,
  "GET": 4 * 3600,
  "GFT": -3 * 3600,
  "GILT": 12 * 3600,
  "GMT": 0 * 3600,
  "GST": 4 * 3600,
  "GYT": -4 * 3600,
  "H": 8 * 3600,
  "HDT": -9 * 3600,
  "HKT": 8 * 3600,
  "HOVST": 8 * 3600,
  "HOVT": 7 * 3600,
  "HST": -10 * 3600,
  "I": 9 * 3600,
  "ICT": 7 * 3600,
  "IDT": 3 * 3600,
  "IOT": 6 * 3600,
  "IRDT": 4.5 * 3600,
  "IRKST": 9 * 3600,
  "IRKT": 8 * 3600,
  "IRST": 3.5 * 3600,
  "IST": 5.5 * 3600,
  "JST": 9 * 3600,
  "K": 10 * 3600,
  "KGT": 6 * 3600,
  "KOST": 11 * 3600,
  "KRAST": 8 * 3600,
  "KRAT": 7 * 3600,
  "KST": 9 * 3600,
  "KUYT": 4 * 3600,
  "L": 11 * 3600,
  "LHDT": 11 * 3600,
  "LHST": 10.5 * 3600,
  "LINT": 14 * 3600,
  "M": 12 * 3600,
  "MAGST": 12 * 3600,
  "MAGT": 11 * 3600,
  "MART": 9.5 * 3600,
  "MAWT": 5 * 3600,
  "MDT": -6 * 3600,
  "MHT": 12 * 3600,
  "MMT": 6.5 * 3600,
  "MSD": 4 * 3600,
  "MSK": 3 * 3600,
  "MST": -7 * 3600,
  "MT": -7 * 3600,
  "MUT": 4 * 3600,
  "MVT": 5 * 3600,
  "MYT": 8 * 3600,
  "N": -1 * 3600,
  "NCT": 11 * 3600,
  "NDT": 2.5 * 3600,
  "NFT": 11 * 3600,
  "NOVST": 7 * 3600,
  "NOVT": 7 * 3600,
  "NPT": 5.5 * 3600,
  "NRT": 12 * 3600,
  "NST": 3.5 * 3600,
  "NUT": -11 * 3600,
  "NZDT": 13 * 3600,
  "NZST": 12 * 3600,
  "O": -2 * 3600,
  "OMSST": 7 * 3600,
  "OMST": 6 * 3600,
  "ORAT": 5 * 3600,
  "P": -3 * 3600,
  "PDT": -7 * 3600,
  "PET": -5 * 3600,
  "PETST": 12 * 3600,
  "PETT": 12 * 3600,
  "PGT": 10 * 3600,
  "PHOT": 13 * 3600,
  "PHT": 8 * 3600,
  "PKT": 5 * 3600,
  "PMDT": -2 * 3600,
  "PMST": -3 * 3600,
  "PONT": 11 * 3600,
  "PST": -8 * 3600,
  "PT": -8 * 3600,
  "PWT": 9 * 3600,
  "PYST": -3 * 3600,
  "PYT": -4 * 3600,
  "Q": -4 * 3600,
  "QYZT": 6 * 3600,
  "R": -5 * 3600,
  "RET": 4 * 3600,
  "ROTT": -3 * 3600,
  "S": -6 * 3600,
  "SAKT": 11 * 3600,
  "SAMT": 4 * 3600,
  "SAST": 2 * 3600,
  "SBT": 11 * 3600,
  "SCT": 4 * 3600,
  "SGT": 8 * 3600,
  "SRET": 11 * 3600,
  "SRT": -3 * 3600,
  "SST": -11 * 3600,
  "SYOT": 3 * 3600,
  "T": -7 * 3600,
  "TAHT": -10 * 3600,
  "TFT": 5 * 3600,
  "TJT": 5 * 3600,
  "TKT": 13 * 3600,
  "TLT": 9 * 3600,
  "TMT": 5 * 3600,
  "TOST": 14 * 3600,
  "TOT": 13 * 3600,
  "TRT": 3 * 3600,
  "TVT": 12 * 3600,
  "U": -8 * 3600,
  "ULAST": 9 * 3600,
  "ULAT": 8 * 3600,
  "UTC": 0 * 3600,
  "UYST": -2 * 3600,
  "UYT": -3 * 3600,
  "UZT": 5 * 3600,
  "V": -9 * 3600,
  "VET": -4 * 3600,
  "VLAST": 11 * 3600,
  "VLAT": 10 * 3600,
  "VOST": 6 * 3600,
  "VUT": 11 * 3600,
  "W": -10 * 3600,
  "WAKT": 12 * 3600,
  "WARST": -3 * 3600,
  "WAST": 2 * 3600,
  "WAT": 1 * 3600,
  "WEST": 1 * 3600,
  "WET": 0 * 3600,
  "WFT": 12 * 3600,
  "WGST": -2 * 3600,
  "WGT": -3 * 3600,
  "WIB": 7 * 3600,
  "WIT": 9 * 3600,
  "WITA": 8 * 3600,
  "WST": 14 * 3600,
  "WT": 0 * 3600,
  "X": -11 * 3600,
  "Y": -12 * 3600,
  "YAKST": 10 * 3600,
  "YAKT": 9 * 3600,
  "YAPT": 10 * 3600,
  "YEKST": 6 * 3600,
  "YEKT": 5 * 3600,
  "Z": 0 * 3600,
}

client_socket = None

def send_message(message, timeout=30):  # 10 seconds timeout
    global client_socket
    try:
        client_socket.settimeout(timeout)
        if isinstance(message, str):
            message = message.encode('utf-8')
        message_length = len(message)
        message_length_bytes = message_length.to_bytes(8, byteorder='big')
        client_socket.send(message_length_bytes)
        # Chunk up into 2048 max
        offset = 0
        while message_length > 0:
          chunk_size = min(4096, message_length)
          chunk = message[offset:offset+chunk_size]
          client_socket.send(chunk)
          offset += chunk_size
          message_length -= chunk_size
        client_socket.settimeout(None)  # Reset to default blocking behavior
    except socket.timeout as e:
        raise FatalScraperError(f"Socket send operation timed out: {e}")
    except socket.error as e:
        raise FatalScraperError(f"Error sending data: {e}")


def recv_message():
    global client_socket
    try:
        message_length_bytes = client_socket.recv(8)
        if not message_length_bytes:
            raise socket.error("No data received for message length")
        message_length = int.from_bytes(message_length_bytes, byteorder='big')

        received_data = b''
        while len(received_data) < message_length:
            remaining_length = message_length - len(received_data)
            chunk_size = min(4096, remaining_length)
            chunk = client_socket.recv(chunk_size)
            if not chunk:
                raise socket.error("Connection closed by server")
            received_data += chunk

        return received_data
    except socket.error as e:
        raise FatalScraperError("Error receiving data: {e}")

# Reconnection logic
def reconnect():
    global client_socket
    max_retries = 100
    retry_delay = 10  # seconds

    client_socket.close()  # Close the existing socket

    for attempt in range(max_retries):
        try:
            print(f"[{timestamp()}] Attempting to reconnect ({attempt+1}/{max_retries})...")
            client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)  # Create a new socket
            client_socket.connect(server_address)
            return
        except socket.error as e:
            print(f"[{timestamp()}] Reconnect attempt failed: {e}")
            time.sleep(retry_delay)
    else:
        print(f"[{timestamp()}] Reconnection failed after {max_retries} attempts. Exiting.")
        sys.exit()


def handle_captcha(session, url):
  driver = webdriver.Chrome()

  driver.get(url)
  for item in session.cookies:
    if item.domain == '.newgrounds.com':
      driver.add_cookie({'name': item.name, 'value': item.value, 'domain': item.domain})
  driver.get(url)
  # Wait any user input
  try:
    WebDriverWait(driver, 1000000, poll_frequency=2).until(lambda x: 'Newgrounds.com: Verifying User' not in x.title)
  except:
    pass
  webdriver_cookies = driver.get_cookies()
  # Copy the cookies from the WebDriver back to the Requests session.
  for cookie in webdriver_cookies:
    session.cookies.set(cookie['name'], cookie['value'], path=cookie['path'])
  userAgent = driver.execute_script("return navigator.userAgent")
  session.headers.update({'User-Agent': userAgent})

  driver.close()


def execute_request(session, url, retry_count=0, wait=2, allow_404=False):
  if retry_count > 8:
    raise FatalScraperError()
  # Mandatory wait period
  to_wait = random.randint(20, 25)
  time.sleep(to_wait)
  # Allow exponential backoff
  wait *= 2
  try:
    response = session.get(url)
    if response.status_code == 200:
      validate_content_length(response)
    elif response.status_code == 404 and allow_404: # Allow 404s for deleted view pages, still need length validation
      validate_content_length(response)
    elif response.status_code == 429:
      if 'Retry-After' in response.headers:
        wait = int(response.headers['Retry-After'])
      time.sleep(wait)
      return execute_request(session, url, retry_count + 1, wait, allow_404)
    elif response.status_code == 503:
      # Captcha response, try and evade for a while if possible
      if wait >= 60:
        handle_captcha(session, url)
      else:
        time.sleep(wait)
      return execute_request(session, url, retry_count + 1, wait, allow_404)
    else:
       raise requests.RequestException(f"Unexpected status code: {response.status_code}")
    return response
  except (requests.RequestException, IOError) as e:
    # It's fucked, reconnect
    raise FatalScraperError(f"Error during request execution: {e}")

def validate_content_length(response):
    expected_length = response.headers.get('Content-Length')
    if expected_length is not None:
        expected_length = int(expected_length)
        print(f"[{timestamp()}] Reading response with length {expected_length}")
        actual_length = response.raw.tell()
        if actual_length < expected_length:
            raise IOError('Incomplete read ({} bytes read, {} more expected)'.format(actual_length, expected_length - actual_length))

def calculate_wait_time(wait, error):
    if isinstance(error, requests.RequestException) and error.response:
        if error.response.status_code == 503:
            return max(wait, 10)
    return min(wait * 2, 60)
  
def get_page_info(response):
  # Parse response text with beautiful soup
  soup = BeautifulSoup(response.text, 'html.parser')
  ng_page_info = {}

  # -- Check if user initiated deletion
  if response.status_code == 404:
    fatal_error_elem = soup.body.find('div', class_='fatal-error')
    if fatal_error_elem:
      message_elem = fatal_error_elem.find_all('div', recursive=False)[1].find('p')
      if message_elem.text == "ERROR — This submission has been deleted.":
        ng_page_info['deleted'] = True
        return ng_page_info
      else:
        return {}
    else:
      return {}
    
  # -- Check if wiki page (Numa Numa)
  if soup.head.find('title').text.startswith('Newgrounds Wiki -'):
    ng_page_info['deleted'] = True
    ng_page_info['wiki_redirect'] = True
    return ng_page_info
    
  # -- Check if user page (James Farr)
  if 'class' in soup.body.attrs and len(soup.body.attrs['class']) > 0 and soup.body.attrs['class'][0] == 'skin-userpage':
    ng_page_info['deleted'] = True
    ng_page_info['user_redirect'] = True
    return ng_page_info

  # -- Check if deleted
  eulogy = soup.body.find('div', id='eulogy', class_='eulogy')
  if eulogy:
    # Find eulogy dates and title
    gravestone = soup.body.find('div', class_='gravestone-text')
    dates_elem = gravestone.find('span')
    # Split elem by <br>
    dates = dates_elem.decode_contents().split('<br/>')
    ng_page_info['date_added'] = parser.parse(dates[0][:-2], tzinfos=whois_timezone_info)
    ng_page_info['date_deleted'] = parser.parse(dates[1][1:], tzinfos=whois_timezone_info)
    ng_page_info['title'] = gravestone.find('h3').text
    ng_page_info['eulogy'] = True
    ng_page_info['deleted'] = True
    return ng_page_info

  # -- Check adult only error
  adult_only_error = soup.body.find('div', class_='adult-only-error')
  if adult_only_error:
    print("Adult only error, please fix auth")
    sys.exit()

  # -- Find basic metadata
  # Find meta elements
  meta_elements = soup.head.find_all('meta', attrs={'property': True, 'content': True})
  for meta in meta_elements:
    if meta['property'].startswith('og:'):
      ng_page_info[meta['property'][3:]] = meta['content']

  ng_page_info['credits'] = []
  ng_page_info['other_credits'] = []

  # -- Find main comments
  author_comments_elem = soup.body.find('div', id='author_comments')
  if author_comments_elem:
    ng_page_info['author_comments'] = author_comments_elem.decode_contents()
  else:
    ng_page_info['author_comments'] = None

  # -- Find author related info
  author_links = soup.body.find_all('div', class_='authorlinks')
  # Sometimes in a ul
  if len(author_links) == 0:
    author_links = soup.body.find_all('ul', class_='authorlinks')
  if len(author_links) > 0:
    for list_elem in author_links[0].find_all('div', recursive=False):
      author_name = list_elem.find('div', class_='item-details-main').find('h4').find('a').text
      author_credits = []
      for role_elem in list_elem.find('div', class_='item-details-meta').find_all('div', recursive=False):
        author_credits.append(role_elem.find('em').text)
      ng_page_info['credits'].append({'name': author_name, 'roles': author_credits})
      pass

  if len(author_links) > 1:
    for list_elem in author_links[1].find_all('h4', class_='detail-title'):
      spans = list_elem.find_all('span')
      if len(spans) > 1:
        # Check for item link
        item_parent = spans[0].find_parent('a', class_='item-link')
        if item_parent:
          ng_page_info['other_credits'].append({'name': spans[0].text.strip(), 'desc': spans[1].text, 'url': item_parent['href']})
        else:
          ng_page_info['other_credits'].append({'name': spans[0].text.strip(), 'desc': spans[1].text})

  # -- Find stats
  stats_element = soup.body.find('div', attrs={'id': 'sidestats'})
  # First elem = views, faves, votes, score. Second elem = uploaded, genre, tags, format
  stats_blocks = stats_element.find_all(recursive=False)
  stats_block_1_values = stats_blocks[0].find_all("dd",recursive=False)
  ng_page_info['views'] = int(stats_block_1_values[0].text.replace(',', ''))
  if len(stats_block_1_values) > 3:
    ng_page_info['faves'] = int(stats_block_1_values[1].find_all("a")[0].text.replace(',', ''))
    try:
      ng_page_info['votes'] = int(stats_block_1_values[2].text.replace(',', ''))
    except:
      ng_page_info['votes'] = 0
    try:
      ng_page_info['score'] = float(stats_block_1_values[3].find_all("span")[0].text)
    except:
      ng_page_info['score'] = -1
  else:
    ng_page_info['faves'] = 0
    try:
      ng_page_info['votes'] = int(stats_block_1_values[1].text.replace(',', ''))
    except:
      ng_page_info['votes'] = 0
    try:
      ng_page_info['score'] = float(stats_block_1_values[2].find_all("span")[0].text)
    except:
      ng_page_info['score'] = -1

  stats_block_2_values = stats_blocks[2].find_all("dd", recursive=False)
  date_added_text = stats_block_2_values[0].text + ' ' + stats_block_2_values[1].text
  ng_page_info['date_added'] = parser.parse(date_added_text, tzinfos=whois_timezone_info)
  
  # Genre
  try:
    ng_page_info['genre'] = stats_blocks[2].find("div").find("dl").find("dd").find("a").text
  except:
    ng_page_info['genre'] = None

  # Tags
  ng_page_info['tags'] = []
  try:
    for list_elem in stats_blocks[3].find("dl").find("dd").find("ul").find_all("li"):
      ng_page_info['tags'].append(list_elem.find("a").text)
  except:
    ng_page_info['tags'] = []

  ng_page_info['rating'] = 'unrated'
  if soup.body.find('h2', class_='rated-e') != None:
    ng_page_info['rating'] = 'e'
  elif soup.body.find('h2', class_='rated-t') != None:
    ng_page_info['rating'] = 't'
  elif soup.body.find('h2', class_='rated-m') != None:
    ng_page_info['rating'] = 'm'
  elif soup.body.find('h2', class_='rated-a') != None:
    ng_page_info['rating'] = 'a'

  # -- Find collection links
  ng_page_info['collections'] = []
  collection_elems = soup.body.find_all('div', class_='item-collection')
  for col_elem in collection_elems:
    try:
      col_link = col_elem["href"]
      col_title = col_elem["title"]
      ng_page_info['collections'].append({
        'url': col_link,
        'title': col_title
      });
    except:
      pass    

  # -- Find flash embed
  # Find all <script> elements
  script_elements = soup.find_all('script')
  # Iterate through the <script> elements and check the content
  for script in script_elements:
    content = script.string
    if content and content.strip().startswith("var embed_controller"):
      json_output = subprocess.check_output(['node', 'parse_embed.js', content], text=True)
      embeds = json.loads(json_output)
      for embed in embeds:
        if embed['description'] == 'Flash Movie' or embed['description'] == 'Flash Game':
          ng_page_info['swf_url'] = embed['url']
          ng_page_info['width'] = embed['width']
          ng_page_info['height'] = embed['height']
          ng_page_info['filesize'] = embed['filesize']
          ng_page_info['params'] = embed['params']

  return ng_page_info

def timestamp():
  return datetime.datetime.now().strftime("%H:%M:%S")
  
def initialize_session(cookies):
    session = None
    if local_ip != "":
      print(f"Binding to {local_ip}")
      session = session_for_src_addr()
    else:
      session = requests.Session()
    session.headers.update({'DNT': '1'})
    session.headers.update({'User-Agent': ua.chrome})
    for cookie in cookies.split("; "):
        name, value = cookie.split("=")
        session.cookies.set(name, value)
    return session

def create_socket_connection(server_address):
    global client_socket
    client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    client_socket.settimeout(30)
    try:
        client_socket.connect(server_address)
    except socket.error as e:
        print(f"Failed to connect to the server: {e}")
        sys.exit()
    return client_socket

def register_client(identifier):
    global client_socket
    send_message(VERSION)
    send_message(identifier)

    data = recv_message()
    if data is None:
        print("Server disconnected while registering.")
        sys.exit()
    signal = data.decode('utf-8')
    if signal != "registered":
        print(f"Server returned unexpected signal: {signal}")
        sys.exit()

def main_loop(session):
    global client_socket
    while True:
        send_ready_signal()
        process_tasks(session)

def send_ready_signal():
    global client_socket
    send_message("ready")
    print(f"[{timestamp()}] Waiting for next task...")

def process_tasks(session):
    global client_socket
    while True:
        data = recv_message()
        if data is None:
            raise FatalScraperError("Disconnection while receiving task")

        signal = data.decode('utf-8')
        if signal == "done":
            break
        elif signal.startswith('download:'):
            process_download_task(signal, session)
        elif signal.startswith('view:'):
            process_view_task(signal, session)

def process_download_task(signal, session):
    global client_socket
    url = signal[len('download:'):]
    print(f"[{timestamp()}] Downloading File: {url}")

    response = execute_request(session, url)
    if response.status_code != 200:
        print("Error retrieving page: HTTP Status Code", response.status_code)
        client_socket.close()
        sys.exit()

    print(f"[{timestamp()}] Returning File")
    send_message(response.content)

def process_view_task(signal, session):
    global client_socket
    url = signal[len('view:'):]
    print(f"[{timestamp()}] Fetching View: {url}")

    response = execute_request(session, url, allow_404=True)
    data = {
        'status': response.status_code,
        'metadata': {}
    }

    if response.status_code in [200, 404]:
        noun = 'Found' if response.status_code == 200 else 'Not Found'
        metadata = get_page_info(response)
        if metadata is None:
            print("Bad auth, exiting, update your cookies")
            sys.exit()

        data['metadata'] = metadata
        print(f"[{timestamp()}] Returning Metadata {response.status_code} - {noun}")
    else:
        print(f"[{timestamp()}] Returning Metadata {response.status_code} - Unexpected Status")

    data_str = json.dumps(data, cls=DateTimeEncoder)
    send_message(data_str)

def read_file(name):
   with open("cookies.txt", "r") as cookie_file:
      return cookie_file.read().strip()
   
def session_for_src_addr() -> requests.Session:
    """
    Create `Session` which will bind to the specified local address
    rather than auto-selecting it.
    """
    session = requests.Session()
    for prefix in ('http://', 'https://'):
        session.get_adapter(prefix).init_poolmanager(
            # those are default values from HTTPAdapter's constructor
            connections=requests.adapters.DEFAULT_POOLSIZE,
            maxsize=requests.adapters.DEFAULT_POOLSIZE,
            # This should be a tuple of (address, port). Port 0 means auto-selection.
            source_address=(local_ip, 0),
        )

    return session

if __name__ == "__main__":
    print("-- Newgrounds Scraper Client --")
    print(f"Dockerized: {IN_DOCKER}")

    cookies = os.environ.get('COOKIES', '') if IN_DOCKER else read_file("cookies.txt")
    server_opts = os.environ.get('MASTER_SERVER', 'localhost:5000').split(":") if IN_DOCKER else sys.argv[1].split(":")
    identifier = os.environ.get('CLIENT_NAME', 'test') if IN_DOCKER else sys.argv[2]
    local_ip = os.environ.get('LOCAL_IP', '')
    if len(sys.argv) > 3:
        local_ip = sys.argv[3]

    if not cookies:
        print("No cookies provided, exiting...")
        sys.exit()

    session = initialize_session(cookies)
    if 'NG_GG_username' not in session.cookies:
        print("Failed to find NG_GG_username cookie, did you load cookies correctly?")
        sys.exit()

    server_address = (server_opts[0], int(server_opts[1]))
    create_socket_connection(server_address)

    while True:
      register_client(identifier)
      try:
          main_loop(session)
      except FatalScraperError as e:
          print(f"[{timestamp()}] Fatal error, reseting connection: {e}")
          reconnect()
      except KeyboardInterrupt:
          print(f"[{timestamp()}] Keyboard interrupt, closing connection.")
          client_socket.close()
          sys.exit()